@@ -1,4 +1,4 @@
-/* Copyright 2007 Matthew Barnes
+/* Copyright 2007, 2008 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -20,6 +20,7 @@
 
 #include <string.h>
 
+#include "gva-categories.h"
 #include "gva-error.h"
 #include "gva-favorites.h"
 #include "gva-mame.h"
@@ -171,6 +172,32 @@
                 "default_ DEFAULT 'no' " \
                 "CHECK (default_ in ('yes', 'no')));"
 
+/* The playback table survives database builds. */
+#define SQL_CREATE_TABLE_PLAYBACK \
+        "CREATE TABLE IF NOT EXISTS playback (" \
+                "name NOT NULL, " \
+                "inode NOT NULL, " \
+                "comment);"
+
+#define SQL_CREATE_VIEW_AVAILABLE \
+        "CREATE VIEW IF NOT EXISTS available AS " \
+                "SELECT *, getcategory(name) AS category, " \
+                "isfavorite(name) AS favorite FROM game " \
+                "WHERE (romset IN ('good', 'best available'))"
+
+#define SQL_DROP_TABLES \
+        "DROP TABLE IF EXISTS mame; " \
+        "DROP TABLE IF EXISTS game; " \
+        "DROP TABLE IF EXISTS biosset; " \
+        "DROP TABLE IF EXISTS rom; " \
+        "DROP TABLE IF EXISTS disk; " \
+        "DROP TABLE IF EXISTS sample; " \
+        "DROP TABLE IF EXISTS chip; " \
+        "DROP TABLE IF EXISTS display; " \
+        "DROP TABLE IF EXISTS control; " \
+        "DROP TABLE IF EXISTS dipvalue; " \
+        "DROP VIEW IF EXISTS available"
+
 #define SQL_INSERT_GAME \
         "INSERT INTO game VALUES (" \
                 "@name, " \
@@ -272,9 +299,6 @@
                 "@name, " \
                 "@default_);"
 
-#define SQL_DELETE_NOT_FOUND \
-        "DELETE FROM game WHERE romset == \"not found\";"
-
 typedef struct _ParserData ParserData;
 
 struct _ParserData
@@ -376,16 +400,18 @@ db_parser_bind_text (sqlite3_stmt *stmt,
                      const gchar *param,
                      const gchar *value)
 {
+        gint index;
         gint errcode;
+        gchar *utf8;
         GError *error = NULL;
 
-        errcode = sqlite3_bind_text (
-                stmt, sqlite3_bind_parameter_index (stmt, param),
-                g_locale_to_utf8 (value, -1, NULL, NULL, &error), -1, g_free);
-
-        /* Handle conversion errors. */
+        index = sqlite3_bind_parameter_index (stmt, param);
+        utf8 = g_locale_to_utf8 (value, -1, NULL, NULL, &error);
         gva_error_handle (&error);
 
+        g_return_if_fail (utf8 != NULL);
+        errcode = sqlite3_bind_text (stmt, index, utf8, -1, g_free);
+
         if (errcode != SQLITE_OK)
         {
                 gva_db_set_error (&error, 0, NULL);
@@ -1186,7 +1212,36 @@ db_create_tables (GError **error)
                 && gva_db_execute (SQL_CREATE_TABLE_CHIP, error)
                 && gva_db_execute (SQL_CREATE_TABLE_DISPLAY, error)
                 && gva_db_execute (SQL_CREATE_TABLE_CONTROL, error)
-                && gva_db_execute (SQL_CREATE_TABLE_DIPVALUE, error);
+                && gva_db_execute (SQL_CREATE_TABLE_DIPVALUE, error)
+                && gva_db_execute (SQL_CREATE_TABLE_PLAYBACK, error)
+                && gva_db_execute (SQL_CREATE_VIEW_AVAILABLE, error);
+}
+
+static void
+db_function_getcategory (sqlite3_context *context,
+                         gint n_values,
+                         sqlite3_value **values)
+{
+#ifdef CATEGORY_FILE
+        const gchar *name;
+        gchar *category;
+        GError *error = NULL;
+
+        g_assert (n_values == 1);
+
+        name = (const gchar *) sqlite3_value_text (values[0]);
+        category = gva_categories_lookup (name, &error);
+        gva_error_handle (&error);
+
+        if (category != NULL)
+                sqlite3_result_text (context, category, -1, SQLITE_TRANSIENT);
+        else
+                sqlite3_result_null (context);
+
+        g_free (category);
+#else
+        sqlite3_result_null (context);
+#endif
 }
 
 static void
@@ -1205,6 +1260,12 @@ db_function_isfavorite (sqlite3_context *context,
                 sqlite3_result_text (context, "no", -1, SQLITE_STATIC);
 }
 
+static void
+db_trace_cb (gpointer unused, const gchar *message)
+{
+        g_debug ("%s", message);
+}
+
 /**
  * gva_db_init:
  * @error: return location for a #GError, or %NULL
@@ -1229,12 +1290,21 @@ gva_db_init (GError **error)
         if (sqlite3_open (filename, &db) != SQLITE_OK)
                 goto fail;
 
+        if (gva_get_debug_flags () & GVA_DEBUG_SQL)
+                sqlite3_trace (db, db_trace_cb, NULL);
+
         errcode = sqlite3_create_function (
                 db, "isfavorite", 1, SQLITE_ANY, NULL,
                 db_function_isfavorite, NULL, NULL);
         if (errcode != SQLITE_OK)
                 goto fail;
 
+        errcode = sqlite3_create_function (
+                db, "getcategory", 1, SQLITE_ANY, NULL,
+                db_function_getcategory, NULL, NULL);
+        if (errcode != SQLITE_OK)
+                goto fail;
+
         return db_create_tables (error);
 
 fail:
@@ -1370,19 +1440,12 @@ gva_db_build (GError **error)
 gboolean
 gva_db_reset (GError **error)
 {
-        const gchar *filename;
-
         g_return_val_if_fail (db != NULL, FALSE);
 
-        filename = gva_db_get_filename ();
+        if (!gva_db_execute (SQL_DROP_TABLES, error))
+                return FALSE;
 
-        sqlite3_close (db);
-        db = NULL;
-
-        if (g_file_test (filename, G_FILE_TEST_EXISTS))
-                g_remove (filename);
-
-        return gva_db_init (error);
+        return db_create_tables (error);
 }
 
 /**
@@ -1395,7 +1458,7 @@ gva_db_reset (GError **error)
  * "BEGIN TRANSACTION".  If an error occurs, it returns %FALSE and sets
  * @error.
  *
- * Returns: %TRUE on succes, %FALSE if an error occurred
+ * Returns: %TRUE on success, %FALSE if an error occurred
  **/
 gboolean
 gva_db_execute (const gchar *sql,
@@ -1418,6 +1481,62 @@ gva_db_execute (const gchar *sql,
         return (errcode == SQLITE_OK);
 }
 
+/**
+ * gva_db_get_table:
+ * @sql: an SQL statement
+ * @result: return location for the result
+ * @rows: return location for the number of rows in the result
+ * @columns: return location for the number of columns in the result
+ * @error: return location for a #GError, or %NULL
+ *
+ * Executes the given SQL statement and returns the results as a string
+ * array (see sqlite3_get_table() for the array layout).  This function
+ * is appropriate for SELECT statements that return a small result set.
+ * If an error occurs, it returns %FALSE and sets @error.
+ *
+ * Use g_strfreev() to free @result.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
+gboolean
+gva_db_get_table (const gchar *sql,
+                  gchar ***result,
+                  gint *rows,
+                  gint *columns,
+                  GError **error)
+{
+        char **table;
+        char *errmsg;
+        gint errcode;
+
+        g_return_val_if_fail (db != NULL, FALSE);
+        g_return_val_if_fail (sql != NULL, FALSE);
+        g_return_val_if_fail (result != NULL, FALSE);
+        g_return_val_if_fail (rows != NULL, FALSE);
+        g_return_val_if_fail (columns != NULL, FALSE);
+
+        errcode = sqlite3_get_table (
+                db, sql, &table, rows, columns, &errmsg);
+
+        if (errcode == SQLITE_OK)
+        {
+                gint length, ii;
+
+                length = (*rows + 1) * (*columns);
+                *result = g_new0 (gchar *, length + 1);
+                for (ii = 0; ii < length; ii++)
+                        (*result)[ii] = g_strdup (table[ii]);
+                sqlite3_free_table (table);
+        }
+        else
+        {
+                gva_db_set_error (error, errcode, errmsg);
+                sqlite3_free (errmsg);
+        }
+
+        return (errcode == SQLITE_OK);
+}
+
 /**
  * gva_db_transaction_begin:
  * @error: return location for a #GError, or %NULL
@@ -1649,5 +1768,5 @@ gva_db_set_error (GError **error,
                 message = sqlite3_errmsg (db);
         }
 
-        g_set_error (error, GVA_SQLITE_ERROR, code, message);
+        g_set_error (error, GVA_SQLITE_ERROR, code, "(SQL) %s", message);
 }
