@@ -1,4 +1,4 @@
-/* Copyright 2007 Matthew Barnes
+/* Copyright 2007, 2008 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -16,6 +16,13 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * SECTION: gva-preferences
+ * @short_description: Preferences Window
+ *
+ * These functions manipulate the Preferences window.
+ **/
+
 #ifndef GVA_PREFERENCES_H
 #define GVA_PREFERENCES_H
 
@@ -23,11 +30,16 @@
 
 G_BEGIN_DECLS
 
-void            gva_preferences_init            (void);
-gboolean        gva_preferences_get_auto_save   (void);
-void            gva_preferences_set_auto_save   (gboolean auto_save);
-gboolean        gva_preferences_get_full_screen (void);
-void            gva_preferences_set_full_screen (gboolean full_screen);
+void           gva_preferences_init             (void);
+gboolean       gva_preferences_get_auto_save    (void);
+void           gva_preferences_set_auto_save    (gboolean auto_save);
+gboolean       gva_preferences_get_full_screen  (void);
+void           gva_preferences_set_full_screen  (gboolean full_screen);
+
+/* Signal Handlers */
+
+void           gva_preferences_close_clicked_cb (GtkWindow *window,
+                                                 GtkButton *button);
 
 G_END_DECLS
 
